@@ -6,21 +6,21 @@
 # Public License. Refer to the included COPYING for full text of license.
 
 # other boot loader distro case
-if [[ ! -r /boot/grub2/grub.cfg ]]; then
+if [[ ! -r /boot/grub2/grub.cfg ]] ; then
     return
 fi
 
 # create grub directory structure
-mkdir -p $v $TMP_DIR/ppc >&2
+mkdir -p $v $TMP_DIR/ppc
 cat >"$TMP_DIR/ppc/bootinfo.txt" <<EOF
 <chrp-boot>
 <description>Relax-and-Recover</description>
 <os-name>Linux</os-name>
-<boot-script>boot &device;:\boot\grub\powerpc.elf</boot-script>
+<boot-script>boot &device;:,\boot\grub\powerpc.elf</boot-script>
 </chrp-boot>
 EOF
 
-mkdir -p $v $TMP_DIR/boot/grub >&2
+mkdir -p $v $TMP_DIR/boot/grub
 cat >"$TMP_DIR/boot/grub/grub.cfg" <<EOF
 set timeout=100
 
@@ -38,4 +38,6 @@ fi
 
 $grub_name-mkimage -O powerpc-ieee1275 -p '()/boot/grub' -o $TMP_DIR/boot/grub/powerpc.elf linux normal iso9660
 
-ISO_FILES=( "${ISO_FILES[@]}" boot=boot ppc=ppc )
+# FIXME: Those additional array elements are no ISO files
+# so what is that special code meant to do?
+ISO_FILES+=( boot=boot ppc=ppc )
